@@ -194,7 +194,17 @@ Print to filehandles:
     $top->print_body(\*STDOUT);
 
 Stringify... note that C<stringify_xx> can also be written C<xx_as_string>;
-the methods are synonymous, and neither form will be deprecated:
+the methods are synonymous, and neither form will be deprecated.
+
+If you set the variable $MIME::Entity::BOUNDARY_DELIMITER to a string,
+that string will be used as the line-end delimiter on output.  If it is not set,
+the line ending will be a newline character (\n)
+
+NOTE that $MIME::Entity::BOUNDARY_DELIMITER only applies to structural
+parts of the MIME data generated by this package and to the Base64
+encoded output; if a part internally uses a different line-end
+delimiter and is output as-is, the line-ending is not changed to match
+$MIME::Entity::BOUNDARY_DELIMITER.
 
     ### Stringify the entire message:
     print $top->stringify;              ### or $top->as_string
@@ -234,7 +244,6 @@ use MIME::Tools qw(:config :msgs :utils);
 use MIME::Head;
 use MIME::Body;
 use MIME::Decoder;
-use IO::Lines;
 
 @ISA = qw(Mail::Internet);
 
@@ -246,7 +255,7 @@ use IO::Lines;
 #------------------------------
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.428";
+$VERSION = "5.515";
 
 ### Boundary counter:
 my $BCount = 0;
@@ -265,10 +274,9 @@ qw(
    );
 
 ### Fallback preamble and epilogue:
-my $DefPreamble = [ "This is a multi-part message in MIME format...\n" ];
+my $DefPreamble = [ "This is a multi-part message in MIME format..." ];
 my $DefEpilogue = [ ];
 
-
 #==============================
 #
 # Utilities, private
@@ -341,6 +349,37 @@ sub new {
     $self;
 }
 
+=item ambiguous_content
+
+I<Instance method.>
+
+Returns true if this entity I<or any of its parts, recursively> has
+a C<MIME::Head> that indicates ambiguous content.
+
+Note carefully the difference between:
+
+    $entity->head->ambiguous_content();
+
+and
+
+    $entity->ambiguous_content();
+
+The first returns true only if this specific entity's headers indicate
+ambiguity.  The second returns true if this entity
+I<or any of its parts, recursively> has headers that indicate ambiguity.
+
+=cut
+sub ambiguous_content {
+    my ($self) = @_;
+
+    return 1 if $self->head->ambiguous_content;
+    return 0 unless $self->is_multipart;
+
+    foreach my $part ($self->parts) {
+        return 1 if $part->ambiguous_content;
+    }
+    return 0;
+}
 
 ###------------------------------
 
@@ -486,6 +525,11 @@ or an array reference (whose elements are joined together to make
 the actual scalar).  The body is opened on the data using
 MIME::Body::InCore.
 
+Note that for text parts, the Data scalar or array is assumed to be
+encoded in a suitable character encoding (as if by C<Encode::encode>)
+rather than a native Perl string.  The encoding you use must, of
+course, match the C<charset> option of the C<MIME-Type> header.
+
 =item Description
 
 I<Optional.>
@@ -571,7 +615,7 @@ sub build {
     $filename = undef if (defined($filename) and $filename eq '');
 
     ### Type-check sanity:
-    if ($type =~ m{^(multipart|message)/}) {
+    if ($type =~ m{^(multipart/|message/(rfc822|partial|external-body|delivery-status|disposition-notification|feedback-report)$)}i) {
 	($encoding =~ /^(|7bit|8bit|binary|-suggest)$/i)
 	    or croak "can't have encoding $encoding for message type $type!";
     }
@@ -768,6 +812,7 @@ or raw data (via read()).
 
 sub body {
 	my ($self, $value) = @_;
+	my $boundary_delimiter = $MIME::Entity::BOUNDARY_DELIMITER || "\n";
 	if (@_ > 1) {      ### setting body line(s)...
 		croak "you cannot use body() to set the encoded contents\n";
 	} else {
@@ -777,7 +822,7 @@ sub body {
 		close($fh);
 		my @ary = split(/\n/, $output);
 		# Each line needs the terminating newline
-		@ary = map { "$_\n" } @ary;
+		@ary = map { "$_$boundary_delimiter" } @ary;
 
 		return \@ary;
 	}
@@ -1384,6 +1429,7 @@ sub sign {
     my %params = @_;
     my $io;
 
+    my $boundary_delimiter = $MIME::Entity::BOUNDARY_DELIMITER || "\n";
     ### If multipart and not attaching, try to sign our first part:
     if ($self->is_multipart and !$params{Attach}) {
 	return $self->parts(0)->sign(@_);
@@ -1430,8 +1476,8 @@ sub sign {
 	my $line;
 	$io = $self->open("w") or croak("open: $!");
 	foreach $line (@body) { $io->print($line) };      ### body data
-	(($body[-1]||'') =~ /\n\Z/) or $io->print("\n");  ### ensure final \n
-	$io->print("-- \n$sig");                          ### separator + sig
+	(($body[-1]||'') =~ /\n\Z/) or $io->print($boundary_delimiter);  ### ensure final \n
+	$io->print("-- $boundary_delimiter$sig");                          ### separator + sig
 	$io->close or croak("close: $!");
 	return 1;         ### done!
     }
@@ -1701,7 +1747,7 @@ sub dump_skeleton {
 I<Instance method, override.>
 Print the entity to the given OUTSTREAM, or to the currently-selected
 filehandle if none given.  OUTSTREAM can be a filehandle, or any object
-that reponds to a print() message.
+that responds to a print() message.
 
 The entity is output as a valid MIME stream!  This means that the
 header is always output first, and the body data (if any) will be
@@ -1765,11 +1811,12 @@ some sort of email handler, it's up to you to save this information.
 use Symbol;
 sub print {
     my ($self, $out) = @_;
+    my $boundary_delimiter = $MIME::Entity::BOUNDARY_DELIMITER || "\n";
     $out = select if @_ < 2;
     $out = Symbol::qualify($out,scalar(caller)) unless ref($out);
 
     $self->print_header($out);   ### the header
-    $out->print("\n");
+    $out->print($boundary_delimiter);
     $self->print_body($out);     ### the "stuff after the header"
 }
 
@@ -1780,7 +1827,7 @@ sub print {
 I<Instance method, override.>
 Print the body of the entity to the given OUTSTREAM, or to the
 currently-selected filehandle if none given.  OUTSTREAM can be a
-filehandle, or any object that reponds to a print() message.
+filehandle, or any object that responds to a print() message.
 
 The body is output for inclusion in a valid MIME stream; this means
 that the body data will be encoded if the header says that it should be.
@@ -1810,29 +1857,37 @@ sub print_body {
     my ($self, $out) = @_;
     $out ||= select;
     my ($type) = split '/', lc($self->mime_type);  ### handle by MIME type
+    my $boundary_delimiter = $MIME::Entity::BOUNDARY_DELIMITER || "\n";
 
     ### Multipart...
     if ($type eq 'multipart') {
 	my $boundary = $self->head->multipart_boundary;
 
 	### Preamble:
-	my $preamble = join('', @{ $self->preamble || $DefPreamble });
-	$out->print("$preamble\n") if ($preamble ne '' or $self->preamble);
+	my $plines = $self->preamble;
+	if (defined $plines) {
+	    # Defined, so output the preamble if it exists (avoiding additional
+	    # newline as per ticket 60931)
+	    $out->print( join('', @$plines) . $boundary_delimiter) if (@$plines > 0);
+	} else {
+	    # Undefined, so use default preamble
+	    $out->print( join('', @$DefPreamble) . $boundary_delimiter . $boundary_delimiter );
+	}
 
 	### Parts:
 	my $part;
 	foreach $part ($self->parts) {
-	    $out->print("--$boundary\n");
+	    $out->print("--$boundary$boundary_delimiter");
 	    $part->print($out);
-	    $out->print("\n");           ### needed for next delim/close
+	    $out->print($boundary_delimiter);           ### needed for next delim/close
 	}
-	$out->print("--$boundary--\n");
+	$out->print("--$boundary--$boundary_delimiter");
 
 	### Epilogue:
 	my $epilogue = join('', @{ $self->epilogue || $DefEpilogue });
 	if ($epilogue ne '') {
 	    $out->print($epilogue);
-	    $out->print("\n") if ($epilogue !~ /\n\Z/);  ### be nice
+	    $out->print($boundary_delimiter) if ($epilogue !~ /\n\Z/);  ### be nice
 	}
     }
 
@@ -1843,7 +1898,7 @@ sub print_body {
 	my $need_sep = 0;
 	my $part;
 	foreach $part ($self->parts) {
-	    $out->print("\n\n") if $need_sep++;
+	    $out->print("$boundary_delimiter$boundary_delimiter") if $need_sep++;
 	    $part->print($out);
 	}
     }
@@ -1874,7 +1929,7 @@ sub print_bodyhandle {
       ### Transparent mode: data is already encoded, so no
       ### need to encode it again
       my $buf;
-      $out->print($buf) while ($IO->read($buf, 2048));
+      $out->print($buf) while ($IO->read($buf, 8192));
     } else {
       ### Get the encoding, defaulting to "binary" if unsupported:
       my $encoding = ($self->head->mime_encoding || 'binary');
@@ -2244,7 +2299,7 @@ L<MIME::Tools>, L<MIME::Head>, L<MIME::Body>, L<MIME::Decoder>, L<Mail::Internet
 =head1 AUTHOR
 
 Eryq (F<eryq@zeegee.com>), ZeeGee Software Inc (F<http://www.zeegee.com>).
-David F. Skoll (dfs@roaringpenguin.com) http://www.roaringpenguin.com
+Dianne Skoll (dianne@skoll.ca)
 
 All rights reserved.  This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
